import NodesRef from './NodesRef'

class SelectQuery {
  constructor () {
    this._component = null
    this._queue = []
    this._queueCb = []
  }

  in (component) {
    this._component = component
    return this
  }

  select (selector) {
    if (typeof selector === 'string') {
      selector = selector.replace('>>>', '>')
    }
    return new NodesRef(selector, this, true)
  }

  selectAll (selector) {
    if (typeof selector === 'string') {
      selector = selector.replace('>>>', '>')
    }
    return new NodesRef(selector, this, false)
  }

  selectViewport () {
    return new NodesRef('html', this, true)
  }

  exec (callback) {
    const res = []
    const handleFields = this._handleFields
    const queueCb = this._queueCb
    this._queue.forEach(item => {
      const { selector, component, single, fields } = item

      let curComponent = document

      if (component && component.$el) {
        curComponent = component.$el
      } else if (component && component.nodeType === 1) {
        curComponent = component
      }

      const selectSelf =
        curComponent === document
          ? false
          : Array
            .from(curComponent.parentNode.querySelectorAll(selector))
            .every(item => item === curComponent)

      if (single) {
        const el = selectSelf ? curComponent : curComponent.querySelector(selector)
        res.push(handleFields(fields, el, selector))
      } else {
        const els = selectSelf
          ? [curComponent]
          : curComponent.querySelectorAll(selector)
        const elsArr = Array.from(els).map(el => handleFields(fields, el, null))
        res.push(elsArr)
      }
    })
    res.forEach((item, idx) => {
      typeof queueCb[idx] === 'function' && queueCb[idx].call(this, item)
    })
    typeof callback === 'function' && callback.call(this, res)
  }

  _handleFields (fields, el, selector) {
    const { id, dataset, rect, size, scrollOffset, properties = [], computedStyle = [] } = fields
    const { left, right, top, bottom, width, height } = el.getBoundingClientRect()

    const res = {}
    const isViewport = selector === 'html'
    if (id) res.id = el.id
    if (dataset) res.dataset = Object.assign({}, el.dataset)
    if (rect) {
      if (isViewport) {
        res.left = 0
        res.right = 0
        res.top = 0
        res.bottom = 0
      } else {
        res.left = left
        res.right = right
        res.top = top
        res.bottom = bottom
      }
    }
    if (size) {
      if (isViewport) {
        res.width = el.clientWidth
        res.height = el.clientHeight
      } else {
        res.width = width
        res.height = height
      }
    }
    if (scrollOffset) {
      res.scrollLeft = el.scrollLeft
      res.scrollTop = el.scrollTop
    }
    properties.forEach(prop => {
      const attr = el.getAttribute(prop)
      if (attr) {
        res[prop] = attr
      }
    })
    if (computedStyle.length) {
      const styles = window.getComputedStyle(el)
      computedStyle.forEach(style => {
        const midLineStyle = style.replace(/[A-Z]/g, m => `-${m.toLowerCase()}`)
        const value = styles.getPropertyValue(midLineStyle)
        if (value) {
          res[style] = value
        }
      })
    }
    return res
  }

  _push (selector, component, single, fields, callback) {
    this._queue.push({
      component,
      selector,
      single,
      fields
    })
    this._queueCb.push(callback)
  }
}

export default SelectQuery
